from typing import List

from dev.coatl.helper.types import AnyStr as AnyStr
from java.lang import Enum

class SpreadsheetVersion(Enum):
    def getLastColumnIndex(self) -> int: ...
    def getLastColumnName(self) -> AnyStr: ...
    def getLastRowIndex(self) -> int: ...
    def getMaxCellStyles(self) -> int: ...
    def getMaxColumns(self) -> int: ...
    def getMaxConditionalFormats(self) -> int: ...
    def getMaxFunctionArgs(self) -> int: ...
    def getMaxRows(self) -> int: ...
    def getMaxTextLength(self) -> int: ...
    @staticmethod
    def getValueOf(name: AnyStr) -> SpreadsheetVersion: ...
    @staticmethod
    def values() -> List[SpreadsheetVersion]: ...
